@@ -216,7 +216,7 @@
 #define   MExtrNoFiles       L"\nNo files to extract"
 #define   MExtrAllOk         L"\nAll OK"
 #define   MExtrTotalErr      L"\nTotal errors: %ld"
-#define   MFileExists        L"\n\n%s already exists. Overwrite it ?"
+#define   MAskReplace        L"\n\nWould you like to replace the existing file %s\n%6s bytes, modified on %s\nwith a new one\n%6s bytes, modified on %s\n"
 #define   MAskOverwrite      L"\nOverwrite %s ?"
 #define   MAskNewName        L"\nEnter new name: "
 #define   MHeaderBroken      L"\nCorrupt header is found"
@@ -344,6 +344,7 @@
 #define   MCreating          L"\nCreating %s"
 #define   MRenaming          L"\nRenaming %s to %s"
 #define   MNTFSRequired      L"\nWrite error: only NTFS file system supports files larger than 4 GB"
+#define   MFAT32Size         L"\nWARNING: FAT32 file system does not support 4 GB or larger files"
 #define   MErrChangeAttr     L"\nWARNING: Cannot change attributes of %s"
 #define   MWrongSFXVer       L"\nERROR: default SFX module does not support RAR %d.%d archives"
 #define   MCannotEncName     L"\nCannot encrypt archive already contained encrypted files"
