@@ -196,7 +196,7 @@ void RemoveNameFromPath(wchar *Path)
 
 
 #if defined(_WIN_ALL) && !defined(SFX_MODULE)
-static void GetAppDataPath(wchar *Path,size_t MaxSize,bool Create)
+bool GetAppDataPath(wchar *Path,size_t MaxSize,bool Create)
 {
   LPMALLOC g_pMalloc;
   SHGetMalloc(&g_pMalloc);
@@ -212,12 +212,8 @@ static void GetAppDataPath(wchar *Path,size_t MaxSize,bool Create)
     if (!Success && Create)
       Success=MakeDir(Path,false,0)==MKDIR_SUCCESS;
   }
-  if (!Success)
-  {
-    GetModuleFileName(NULL,Path,(DWORD)MaxSize);
-    RemoveNameFromPath(Path);
-  }
   g_pMalloc->Free(ppidl);
+  return Success;
 }
 #endif
 
@@ -237,7 +233,11 @@ void GetRarDataPath(wchar *Path,size_t MaxSize,bool Create)
   }
 
   if (*Path==0 || !FileExist(Path))
-    GetAppDataPath(Path,MaxSize,Create);
+    if (!GetAppDataPath(Path,MaxSize,Create))
+    {
+      GetModuleFileName(NULL,Path,(DWORD)MaxSize);
+      RemoveNameFromPath(Path);
+    }
 }
 #endif
 
@@ -436,9 +436,12 @@ void MakeNameUsable(wchar *Name,bool Extended)
 #ifndef _UNIX
     if (s-Name>1 && *s==':')
       *s='_';
+#if 0  // We already can create such files.
     // Remove ' ' and '.' before path separator, but allow .\ and ..\.
-    if ((*s==' ' || *s=='.' && s>Name && !IsPathDiv(s[-1]) && s[-1]!='.') && IsPathDiv(s[1]))
+    if (IsPathDiv(s[1]) && (*s==' ' || *s=='.' && s>Name &&
+        !IsPathDiv(s[-1]) && (s[-1]!='.' || s>Name+1 && !IsPathDiv(s[-2]))))
       *s='_';
+#endif
 #endif
   }
 }
