@@ -1,6 +1,6 @@
 
 
-static bool UnixSymlink(const char *Target,const wchar *LinkName)
+static bool UnixSymlink(const char *Target,const wchar *LinkName,RarTime *ftm,RarTime *fta)
 {
   CreatePath(LinkName,true);
   DelFile(LinkName);
@@ -17,9 +17,15 @@ static bool UnixSymlink(const char *Target,const wchar *LinkName)
     }
     return false;
   }
-  // We do not set time of created symlink, because utime changes
-  // time of link target and lutimes is not available on all Linux
-  // systems at the moment of writing this code.
+#ifdef USE_LUTIMES
+  struct timeval tv[2];
+  tv[0].tv_sec=fta->GetUnix();
+  tv[0].tv_usec=long(fta->GetRaw()%10000000/10);
+  tv[1].tv_sec=ftm->GetUnix();
+  tv[1].tv_usec=long(ftm->GetRaw()%10000000/10);
+  lutimes(LinkNameA,tv);
+#endif
+
   return true;
 }
 
@@ -42,7 +48,7 @@ bool ExtractUnixLink30(ComprDataIO &DataIO,Archive &Arc,const wchar *LinkName)
     if (!DataIO.UnpHash.Cmp(&Arc.FileHead.FileHash,Arc.FileHead.UseHashKey ? Arc.FileHead.HashKey:NULL))
       return true;
 
-    return UnixSymlink(Target,LinkName);
+    return UnixSymlink(Target,LinkName,&Arc.FileHead.mtime,&Arc.FileHead.atime);
   }
   return false;
 }
@@ -62,5 +68,5 @@ bool ExtractUnixLink50(const wchar *Name,FileHeader *hd)
       return false;
     DosSlashToUnix(Target,Target,ASIZE(Target));
   }
-  return UnixSymlink(Target,Name);
+  return UnixSymlink(Target,Name,&hd->mtime,&hd->atime);
 }
